package com.muimi.mail.model;

import java.io.File;

import javax.mail.Flags;
import javax.mail.Folder;
import javax.mail.Message;
import javax.mail.MessagingException;
import javax.mail.event.MessageCountListener;

import jp.gr.java_conf.roadster.net.pop.POP3Folder;

import org.eclipse.ui.IMemento;

import com.muimi.mail.MailPlugin;

/**
 * [J̃[tH_킷NX
 * @author Shinji Miyamoto
 */
public class LocalFolder {

	private Account account;

	private String label;

	private String folderPath;

	private Folder mailFolder;


	public static LocalFolder createInBox(Account account){
		LocalFolder folder = new LocalFolder(account);
		folder.label = "INBOX";
		folder.folderPath = "INBOX";
		return folder;
	}
	
	public static LocalFolder createOutBox(Account account){
		LocalFolder folder = new LocalFolder(account);
		folder.label = "OUTBOX";
		folder.folderPath = "OUTBOX";
		folder.realize();
		return folder;
	}
	
	public static LocalFolder createTrash(Account account){
		LocalFolder folder = new LocalFolder(account);
		folder.label = "TRASH";
		folder.folderPath = "TRASH";
		folder.realize();
		return folder;
	}
	
	
	
	public LocalFolder(Account account){
		this.account = account;
	}
	
	/**
	 * t@CVXeɃtH_쐬B
	 */
	private void realize(){
		File dir = account.getLocalFolderDir(folderPath);
		if(!dir.exists()){
			if(!dir.mkdirs()){
				throw new IllegalStateException("fail to crate OUTBOX : " + dir.getAbsolutePath());
			}
		}
	}


	public Message[] getMessages() throws MessagingException{
		return getFolder(Folder.READ_WRITE).getMessages();
	}

	public Message[] getRemoteMessages() throws MessagingException{
		mailFolder = account.getRemoteFolder(folderPath);
		mailFolder.open(Folder.READ_WRITE);//TODO here open
		return mailFolder.getMessages();
	}

	private Folder getFolder(int flg) throws MessagingException{
		mailFolder = account.getFolder(folderPath);
		mailFolder.open(flg);//TODO here open
		return mailFolder;
	}

	
	public void close() throws MessagingException{
		if(mailFolder != null){
			if(mailFolder.isOpen()){
				System.out.println("close");
				mailFolder.close(true);
			}
		}
	}

	public void addMessageCountListener(MessageCountListener listener){
		mailFolder.addMessageCountListener(listener);
	}



	public boolean isInBox(){
		return folderPath.equals("INBOX");
	}
	
	public boolean isOutBox(){
		return folderPath.equals("OUTBOX");
	}
	
	public boolean isTrash(){
		return folderPath.equals("TRASH");
	}
	
	


	public String getLabel(){
		return label;
	}

	
	public Account getAccount(){
		return account;
	}
	


	public void addMessage(Message message){
		try {
			POP3Folder folder = (POP3Folder)getFolder(Folder.READ_WRITE);
			folder.appendMessages(new Message[]{message});
		} catch (MessagingException e) {
			MailPlugin.logError("fail to store sent file", e);
		}
	}


	public void deleteMessage(Message message) throws MessagingException{
		if(!isTrash()){
			account.getTrash().addMessage(message);
		}
		message.setFlag(Flags.Flag.DELETED, true);
	}




	public void persist(IMemento memento){
		memento.putString("label", label);
		memento.putString("folderPath", folderPath);
	}

	
	public void restore(IMemento memento){
		label = memento.getString("label");
		folderPath = memento.getString("folderPath");
	}










//	public void addMessageCountSubject(Folder folder){
//		folder.addMessageCountListener(new TMessageCountListener());
//	}
//
//	public void addMessageView(MessageView view){
//		this.view = view;
//	}
//
//	class TMessageCountListener implements MessageCountListener {
//
//		public void messagesAdded(MessageCountEvent event) {
//			if(view != null){
//				view.addMessages(event.getMessages());
//			}
//		}
//
//		public void messagesRemoved(MessageCountEvent event) {
//		}
//	}
//	
	



}
