package com.muimi.mail.ui.editor.content;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.eclipse.jface.text.TextViewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.StyleRange;
import org.eclipse.swt.custom.StyledText;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.events.MouseListener;
import org.eclipse.swt.events.MouseMoveListener;
import org.eclipse.swt.graphics.Cursor;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Display;

import com.muimi.mail.MailPlugin;
import com.muimi.mail.PluginPreference;
import com.muimi.mail.PluginResources;
import com.muimi.mail.internal.InternalBridge;
import com.muimi.mail.ui.editor.MessageModel;

/**
 * [GfB^̖{̃GfB^
 * AStyledText𑀍
 * JDK1.4̐K\Cu𗘗p(JDK1.4ˑ͂̂)
 * @author Shinji Miyamoto
 */
public class ContentEditor_Old implements IContentEditor{

	private MessageModel messageModel;

	private TextViewer viewer;


	private StyledText contentText;

	private List ranges = new ArrayList();

	private Pattern hyperlinkPattern =
		Pattern.compile("http://[a-zA-Z\\./0-9%]*");
	private Pattern mailAddrPattern =
		Pattern.compile(
			"[0-9,A-Z,a-z][0-9,a-z,A-Z,_,\\.,-]+@[0-9,A-Z,a-z][0-9,a-z,A-Z,_,\\.,-]+");
	private Pattern replyPattern 
		= Pattern.compile(">.*");
	private Pattern viewPattern 
		= Pattern.compile("org\\.eclipse\\.[a-zA-Z\\.0-9%]+");

	//private Pattern mailAddrPattern = Pattern.compile("[^ \t]\\w+@\\w+[^ \t]");

	private Cursor handCursor;
	private Cursor arrowCursor;
	private Display display;

	public ContentEditor_Old() {
	}

	public ContentEditor_Old(MessageModel messageModel) {
		this.messageModel = messageModel;
	}

	public Control createContent(Composite parent, boolean readOnly) {


		display = Display.getCurrent();
		handCursor = new Cursor(display, SWT.CURSOR_HAND);
		arrowCursor = new Cursor(display, SWT.CURSOR_ARROW);

		Composite c = new Composite(parent, SWT.NULL);
		c.setLayout(new FillLayout());

		if (readOnly) {
			contentText =
				new StyledText(
					c,
					SWT.READ_ONLY | SWT.BORDER | SWT.MULTI | SWT.V_SCROLL);
			contentText.setBackground(PluginResources.COL_BG);
		} else {
			contentText =
				new StyledText(c, SWT.BORDER | SWT.MULTI | SWT.V_SCROLL);
		}

		contentText.addModifyListener(new MyModifyListener());
		contentText.addMouseMoveListener(new MyMouseMoveListener());
		contentText.addMouseListener(new MyMouseListener());

		return c;
	}

	public void setText(String text) {
		if (text != null) {
			contentText.setText(text);
		}
	}

	public String getText() {
		return contentText.getText();
	}

	class MyMouseMoveListener implements MouseMoveListener {

		public void mouseMove(MouseEvent e) {
			Point p = new Point(e.x, e.y);
			if (getActionRange(p) != null) {
				contentText.setCursor(handCursor);
			} else {
				contentText.setCursor(arrowCursor);
			}
		}
	}

	class MyMouseListener implements MouseListener {

		public void mouseDoubleClick(MouseEvent e) {
			Point p = new Point(e.x, e.y);
			ActionStyleRange range = getActionRange(p);
			if (range != null) {
				range.run();
			}
		}

		public void mouseDown(MouseEvent e) {
		}

		public void mouseUp(MouseEvent e) {
		}
	}

	private ActionStyleRange getActionRange(Point point) {
		try {
			int offset = contentText.getOffsetAtLocation(point);
			for (Iterator iter = ranges.iterator(); iter.hasNext();) {
				ActionStyleRange range = (ActionStyleRange) iter.next();
				if (range.start <= offset
					&& offset <= range.start + range.length) {
					return range;
				}
			}
		} catch (IllegalArgumentException ignore) {
			//out of the document range
		}
		return null;
	}




	class MyModifyListener implements ModifyListener {

		public void modifyText(ModifyEvent e) {


			//TODO ɖS͂炢ƁBBB

			contentText.setStyleRanges(new StyleRange[0]);
			ranges.clear();

			checkHilight();

			//System.out.println("================");
			//			StyleRange[] ranges = contentText.getStyleRanges();
			//			for(int i=0; i<ranges.length; i++){
			//				System.out.println(ranges[i]);
			//			}

			//try{
			//			Point point = contentText.getSelection();
			//			int offset = point.x;
			//
			//			int line = contentText.getLineAtOffset(offset);
			//
			//			int start = contentText.getOffsetAtLine(line);
			//			int limit = contentText.getLineCount();
			//			int end;
			//			if(start == limit){
			//				end = contentText.getCharCount();
			//			}else{
			//				end = contentText.getOffsetAtLine(line+1);				
			//			}
			//
			//
			//			String text = contentText.getText(start, end-1);
			//			System.out.println(text);
			//}catch(Exception ex){
			//	System.out.println(ex);
			//}

		}
	}

	private void checkHilight() {

		String content = contentText.getText();

		Matcher matcher = hyperlinkPattern.matcher(content);

		while (matcher.find()) {
			String line = matcher.group();
			int index = content.indexOf(line);
			HyperlinkRange range = new HyperlinkRange();
			range.start = index;
			range.length = line.length();
			range.fontStyle = SWT.BOLD;
			range.foreground = PluginResources.COL_HYPERLINK;
			contentText.setStyleRange(range);
			range.keyword = line;
			ranges.add(range);
		}

		matcher = mailAddrPattern.matcher(content);

		while (matcher.find()) {
			String line = matcher.group();
			int index = content.indexOf(line);
			MailRange range = new MailRange();
			range.start = index;
			range.length = line.length();
			range.fontStyle = SWT.BOLD;
			range.foreground = PluginResources.COL_MAIL;
			contentText.setStyleRange(range);
			range.keyword = line;
			ranges.add(range);
		}

		matcher = viewPattern.matcher(content);
		while (matcher.find()) {
			String line = matcher.group();
			int index = content.indexOf(line);
			EclipseViewRange range = new EclipseViewRange();
			range.start = index;
			range.length = line.length();
			range.fontStyle = SWT.BOLD;
			range.foreground = PluginResources.COL_ECLIPSE;
			contentText.setStyleRange(range);
			range.keyword = line;
			ranges.add(range);
		}

		matcher = replyPattern.matcher(content);
		while (matcher.find()) {
			String line = matcher.group();
			int index = content.indexOf(line);
			StyleRange range = new StyleRange();
			range.start = index;
			range.length = line.length();
			range.fontStyle = SWT.BOLD;
			range.foreground = PluginResources.COL_REPLY;
			contentText.setStyleRange(range);
		}

	}

	public abstract class ActionStyleRange extends StyleRange {
		String keyword;
		abstract void run();
	}

	public class HyperlinkRange extends ActionStyleRange {
		public void run() {
			boolean win32 = SWT.getPlatform().equals("win32");
			boolean useEmbedded = PluginPreference.isUseEmbeddedBrowzer();
			if (win32 && useEmbedded) {
				InternalBridge.openInBrowzer(keyword);
			} else {
				String browzerPath = PluginPreference.getBworzerPath();
				try {
					Runtime.getRuntime().exec(
						new String[] { browzerPath, keyword });
				} catch (IOException e) {
					MailPlugin.logError("error in open browzer", e);
				}
			}
		}
	}

	public class MailRange extends ActionStyleRange {
		public void run() {
			MessageModel newMessageModel =
				messageModel.createMessageAsContext(keyword);
			MailPlugin.getDefault().openMessage(newMessageModel);
		}
	}

	public class EclipseViewRange extends ActionStyleRange {
		public void run() {
			MailPlugin.showView(keyword);
		}
	}

}
